/***************************************************
//Web: http://www.buydisplay.com
EastRising Technology Co.,LTD
Examples for ER-TFT3.99-1 with ER-PCBA5981-1 Capacitive touch screen test
Display is Hardward SPI 4-Wire SPI Interface and 5V Power Supply,CTP is I2C interface.
Tested and worked with:
Works with Arduino 2.3.4 IDE  
NOTE: test OK:ESP32-WROOM-32 
****************************************************/

/*
  ==TFT Hardware SPI to ESP32  ==
    TFT   =>    ESP32
  *1,2. GND       ->    GND
  *3,4. VCC       ->    5V    3.3V OR 5V is optional, depending on the voltage of the module purchased 
  *5. CS        ->       5
  *6. MISO        ->    19  
  *7. MOSI       ->    23  
  *8. SCK       ->    18  
  *9. BL        ->     3.3V  (HIGH:ON  LOW:OFF)
  *11. RES       ->    16   
  *12. CTPINT     ->    17
  *13. CTPRST     ->    15
  *14. CTPSDA    ->    21
  *15. CTPSCL    ->    22 

  *16. CS        ->    11 
  *18. SDI       ->    12 
  *20. SCK       ->    13 
*/

#include <SPI.h>
#include <Wire.h>
#include "LCD.h"
#include "LCD_init.h"
#include <stdint.h>
#include "Arduino.h"
#include "Print.h"

uint8_t addr  = 0x5A;  //CTP IIC ADDRESS

#define CST3240_RST 15
#define CST3240_INT   17 

#define HYN_REG_MUT_DEBUG_INFO_MODE             0xD101
#define HYN_REG_MUT_DEEP_SLEEP_MODE    			0xD105
#define HYN_REG_MUT_NORMAL_MODE                0xD109
#define HYN_REG_MUT_DEBUG_INFO_IC_CHECKSUM             0xD208
#define HYN_REG_MUT_DEBUG_INFO_FW_VERSION              0xD204
#define HYN_REG_MUT_DEBUG_INFO_RES_Y            	   0xD1FA
#define HYN_REG_MUT_DEBUG_INFO_RES_X            	   0xD1F8
#define GTP_READ_COOR_ADDR    0xD000

struct _ts_event
{
    uint8_t     touch_pressure;	
    uint8_t     touch_point;
	
};
struct _ts_event ts_event; 


struct TouchLocation
{
  uint16_t x;
  uint16_t y;
};

TouchLocation touchLocations[5];

	
uint8_t buf[30];


void inttostr(uint16_t value,uint8_t *str);
void writeCST3240TouchRegister( uint16_t regAddr,uint8_t *val, uint16_t cnt);
uint8_t readCST3240TouchAddr( uint16_t regAddr, uint8_t * pBuf, uint8_t len );
uint8_t readCST3240TouchLocation( void);



void writeCST3240TouchRegister( uint16_t regAddr,uint8_t *val, uint16_t cnt)
{	uint16_t i=0;
  Wire.beginTransmission(addr);
   Wire.write( regAddr>>8 );  // register 0
  Wire.write( regAddr);  // register 0 
	for(i=0;i<cnt;i++,val++)//data
	{		
          Wire.write( *val );  // value
	}
  uint8_t retVal = Wire.endTransmission(); 
}



uint8_t readCST3240TouchAddr( uint16_t regAddr, uint8_t * pBuf, uint8_t len )
{
  Wire.beginTransmission(addr);
  Wire.write( regAddr>>8 );  // register 0
  Wire.write( regAddr);  // register 0  
  uint8_t retVal = Wire.endTransmission();
  
  uint8_t returned = Wire.requestFrom(addr, len);    // request 1 bytes from slave device #2
  
  uint8_t i;
  for (i = 0; (i < len) && Wire.available(); i++)
  
  {
    pBuf[i] = Wire.read();
  }
  
  return i;
}

uint8_t readCST3240TouchLocation( void)
{
  uint8_t retVal;
  uint8_t i;
  uint8_t k;
  uint8_t  ss[1];
  do
  {    
     ss[0]=0;
      readCST3240TouchAddr(GTP_READ_COOR_ADDR,buf,28);
    ts_event.touch_point = buf[5]&0x0F;
    if (ts_event.touch_point == 0) break; // no points detected
   ts_event.touch_pressure= buf[4];

    
   if  (ts_event.touch_point==1)                       
        {
          touchLocations[0].x =(uint16_t)(buf[1])<<4 | (uint16_t)buf[3]>>4&0x0F;;  
          touchLocations[0].y = (uint16_t)(buf[2])<<4 | (uint16_t)buf[3]&0x0F;  
        }   
   if  (ts_event.touch_point==2)                   
        {touchLocations[0].x =(uint16_t)(buf[1])<<4 | (uint16_t)buf[3]>>4&0x0F;;  
          touchLocations[0].y = (uint16_t)(buf[2])<<4 | (uint16_t)buf[3]&0x0F;
          touchLocations[1].x =(uint16_t)(buf[8] )<<4 | (uint16_t)buf[10]>>4&0x0F;;
          touchLocations[1].y =(uint16_t)(buf[9] )<<4 | (uint16_t)buf[10]&0x0F;
        }  
    if  (ts_event.touch_point==3)                   
        {touchLocations[0].x =(uint16_t)(buf[1])<<4 | (uint16_t)buf[3]>>4&0x0F;;  
          touchLocations[0].y = (uint16_t)(buf[2])<<4 | (uint16_t)buf[3]&0x0F;
          touchLocations[1].x =(uint16_t)(buf[8] )<<4 | (uint16_t)buf[10]>>4&0x0F;;
          touchLocations[1].y =(uint16_t)(buf[9] )<<4 | (uint16_t)buf[10]&0x0F;        
          touchLocations[2].x =(uint16_t)(buf[13] )<<4 | (uint16_t)buf[15]>>4&0x0F;
          touchLocations[2].y = (uint16_t)(buf[14] )<<4 | (uint16_t)buf[15]&0x0F;
        }    
    if  (ts_event.touch_point==4)                   
        {touchLocations[0].x =(uint16_t)(buf[1])<<4 | (uint16_t)buf[3]>>4&0x0F;;  
          touchLocations[0].y = (uint16_t)(buf[2])<<4 | (uint16_t)buf[3]&0x0F;
          touchLocations[1].x =(uint16_t)(buf[8] )<<4 | (uint16_t)buf[10]>>4&0x0F;;
          touchLocations[1].y =(uint16_t)(buf[9] )<<4 | (uint16_t)buf[10]&0x0F;        
          touchLocations[2].x =(uint16_t)(buf[13] )<<4 | (uint16_t)buf[15]>>4&0x0F;
          touchLocations[2].y = (uint16_t)(buf[14] )<<4 | (uint16_t)buf[15]&0x0F;
          touchLocations[3].x =(uint16_t)(buf[18] )<<4 | (uint16_t)buf[20]>>4&0x0F; 
          touchLocations[3].y =(uint16_t)(buf[19] )<<4 | (uint16_t)buf[20]&0x0F;
        }   
     if  (ts_event.touch_point==5)                   
        {touchLocations[0].x =(uint16_t)(buf[1])<<4 | (uint16_t)buf[3]>>4&0x0F;;  
          touchLocations[0].y = (uint16_t)(buf[2])<<4 | (uint16_t)buf[3]&0x0F;
          touchLocations[1].x =(uint16_t)(buf[8] )<<4 | (uint16_t)buf[10]>>4&0x0F;;
          touchLocations[1].y =(uint16_t)(buf[9] )<<4 | (uint16_t)buf[10]&0x0F;        
          touchLocations[2].x =(uint16_t)(buf[13] )<<4 | (uint16_t)buf[15]>>4&0x0F;
          touchLocations[2].y = (uint16_t)(buf[14] )<<4 | (uint16_t)buf[15]&0x0F;
          touchLocations[3].x =(uint16_t)(buf[18] )<<4 | (uint16_t)buf[20]>>4&0x0F; 
          touchLocations[3].y =(uint16_t)(buf[19] )<<4 | (uint16_t)buf[20]&0x0F;
          touchLocations[4].x =(uint16_t)(buf[23] )<<4 | (uint16_t)buf[25]>>4&0x0F;
          touchLocations[4].y =(uint16_t)(buf[24] )<<4 | (uint16_t)buf[25]&0x0F;
        }  
  
    retVal = ts_event.touch_point;
    
  } while (0);
  
  return retVal;
}

void inttostr(uint16_t value,uint8_t *str)
{
	str[0]=value/1000+0x30;
	str[1]=value%1000/100+0x30;
	str[2]=value%1000%100/10+0x30;
	str[3]=value%1000%100%10+0x30;

}



void setup() {uint8_t ss[4];
  Serial.begin(9600);
 Wire.setClock(400000);
  Wire.begin();        // join i2c bus (address optional for master)   


   delay(100);
  ER_TFT.Parallel_Init();
  ER_TFT.HW_Reset();
  ER_TFT.System_Check_Temp();
  delay(100);
  while(ER_TFT.LCD_StatusRead()&0x02);
  ER_TFT.initial();
  ST7701S_Initial(); 
  ER_TFT.Display_ON();

  ER_TFT.Select_Main_Window_16bpp();
  ER_TFT.Main_Image_Start_Address(layer1_start_addr);        
  ER_TFT.Main_Image_Width(LCD_XSIZE_TFT);
  ER_TFT.Main_Window_Start_XY(0,0);
  ER_TFT.Canvas_Image_Start_address(0);
  ER_TFT.Canvas_image_width(LCD_XSIZE_TFT);
  ER_TFT.Active_Window_XY(0,0);
  ER_TFT.Active_Window_WH(LCD_XSIZE_TFT,LCD_YSIZE_TFT); 
  
  ER_TFT.Foreground_color_65k(Black);
  ER_TFT.Line_Start_XY(0,0);
  ER_TFT.Line_End_XY(LCD_XSIZE_TFT-1,LCD_YSIZE_TFT-1);
  ER_TFT.Start_Square_Fill(); 
  
   ER_TFT.Foreground_color_65k(White);
  ER_TFT.Background_color_65k(Black);
  ER_TFT.CGROM_Select_Internal_CGROM();  
  ER_TFT.Font_Select_8x16_16x16();
  ER_TFT.Goto_Text_XY(0,0); 
  ER_TFT.Show_String( "www.buydisplay.com");   
  ER_TFT.Goto_Text_XY(0,16); 
  ER_TFT.Show_String( "Capacitive touch screen tese"); 
 		ER_TFT.Goto_Text_XY(LCD_XSIZE_TFT-48,LCD_YSIZE_TFT-20);
		ER_TFT.Show_String("Clear");   
		ER_TFT.Goto_Text_XY(15,LCD_YSIZE_TFT-20);
		ER_TFT.Show_String("Exit");   

    
    pinMode(CST3240_RST, OUTPUT); 
    pinMode     (CST3240_INT, INPUT);
    digitalWrite(CST3240_RST, LOW);
    delay(20);
    digitalWrite(CST3240_RST, HIGH);
    delay(50);  
    writeCST3240TouchRegister(HYN_REG_MUT_NORMAL_MODE ,0 ,0);  
 
}

 uint8_t flag = 1;
void loop() {
    static uint16_t w = LCD_XSIZE_TFT;
  static uint16_t h = LCD_YSIZE_TFT; 
   unsigned int i;
   double float_data;  
 
 
  while(flag) 
  { 
      pinMode     (CST3240_INT, INPUT); 
       uint8_t st=digitalRead(CST3240_INT);       
      if(!st)    //Hardware touch interrupt
    {  
      Serial.println("Touch: ");
      
        readCST3240TouchLocation();
                
        for (i = 0; i < ts_event.touch_point; i++)
          {  
            
	   if ((touchLocations[0].x<85) &&(touchLocations[0].y>920)) flag=0;     
            if (((touchLocations[0].x)>320) &&((touchLocations[0].y)>920))
              {    ER_TFT.Foreground_color_65k(Black);
                  ER_TFT.Line_Start_XY(0,0);
                  ER_TFT.Line_End_XY(LCD_XSIZE_TFT-1,LCD_YSIZE_TFT-1);
                  ER_TFT.Start_Square_Fill(); 
                  
                   ER_TFT.Foreground_color_65k(White);
                  ER_TFT.Background_color_65k(Black);
                  ER_TFT.CGROM_Select_Internal_CGROM();  
                  ER_TFT.Font_Select_8x16_16x16();
                  ER_TFT.Goto_Text_XY(0,0); 
                  ER_TFT.Show_String( "www.buydisplay.com");   
                  ER_TFT.Goto_Text_XY(0,16); 
                  ER_TFT.Show_String( "Capacitive touch screen test"); 
		  ER_TFT.Goto_Text_XY(LCD_XSIZE_TFT-48,LCD_YSIZE_TFT-20);
		  ER_TFT.Show_String("Clear");   
		  ER_TFT.Goto_Text_XY(15,LCD_YSIZE_TFT-20);
		  ER_TFT.Show_String("Exit");                  
              }  
             
          else{ 
         snprintf((char*)buf,sizeof(buf),"(%3d,%3d)",touchLocations[i].x,touchLocations[i].y);                 
            const  char *str=(const char *)buf;
             ER_TFT.Foreground_color_65k(White>>(i*3));  
            ER_TFT.Text_Mode();
            ER_TFT.Goto_Text_XY(50,80+16*i);
            ER_TFT.LCD_CmdWrite(0x04);
            while(*str != '\0')
            {
            ER_TFT.LCD_DataWrite(*str);
            ER_TFT.Check_Mem_WR_FIFO_not_Full();      
            ++str; 
            } 
            ER_TFT.Check_2D_Busy();
            ER_TFT.Graphic_Mode(); //back to graphic mode;
 
             if(i==0)  ER_TFT.DrawCircle_Fill(touchLocations[i].x,touchLocations[i].y, ts_event.touch_pressure/12, Red);  
            else if(i==1)  ER_TFT.DrawCircle_Fill(touchLocations[i].x,touchLocations[i].y, ts_event.touch_pressure/12, Green); 
            else if(i==2)  ER_TFT.DrawCircle_Fill(touchLocations[i].x,touchLocations[i].y,ts_event.touch_pressure/12, Blue);        
            else if(i==3)  ER_TFT.DrawCircle_Fill(touchLocations[i].x,touchLocations[i].y, ts_event.touch_pressure/12, Cyan); 
             else if(i==4)  ER_TFT.DrawCircle_Fill(touchLocations[i].x,touchLocations[i].y, ts_event.touch_pressure/12, Yellow);     
           
        
  
          }
        }
     }

    
  }
 
  
  ////////Drawing
  ER_TFT.Select_Main_Window_16bpp();
  ER_TFT.Main_Image_Start_Address(layer1_start_addr);				
  ER_TFT.Main_Image_Width(LCD_XSIZE_TFT);
  ER_TFT.Main_Window_Start_XY(0,0);
  ER_TFT.Canvas_Image_Start_address(layer1_start_addr);
  ER_TFT.Canvas_image_width(LCD_XSIZE_TFT);
  ER_TFT.Active_Window_XY(0,0);
  ER_TFT.Active_Window_WH(LCD_XSIZE_TFT,LCD_YSIZE_TFT);	

  ER_TFT.Foreground_color_65k(Black);
  ER_TFT.Line_Start_XY(0,0);
  ER_TFT.Line_End_XY(LCD_XSIZE_TFT-1,LCD_YSIZE_TFT-1);
  ER_TFT.Start_Square_Fill();

  for(i=0;i<=LCD_XSIZE_TFT/2-10;i+=8)
  {ER_TFT.Foreground_color_65k(Red);
  ER_TFT.Line_Start_XY(0+i,0+i);
  ER_TFT.Line_End_XY(LCD_XSIZE_TFT-1-i,LCD_YSIZE_TFT-1-i);
  ER_TFT.Start_Square();
  delay(10);
  }

  for(i=0;i<=LCD_XSIZE_TFT/2-10;i+=8)
  {ER_TFT.Foreground_color_65k(Black);
  ER_TFT.Line_Start_XY(0+i,0+i);
  ER_TFT.Line_End_XY(LCD_XSIZE_TFT-1-i,LCD_YSIZE_TFT-1-i);
  ER_TFT.Start_Square();
  delay(10);
  }
 delay(100);
///////////////////////////Square Of Circle
  ER_TFT.Foreground_color_65k(Black);
  ER_TFT.Line_Start_XY(0,0);
  ER_TFT.Line_End_XY(LCD_XSIZE_TFT-1,LCD_YSIZE_TFT-1);
  ER_TFT.Start_Square_Fill();

  for(i=0;i<=LCD_XSIZE_TFT/2-10;i+=8)
  {ER_TFT.Foreground_color_65k(Green);
  ER_TFT.Line_Start_XY(0+i,0+i);
  ER_TFT.Line_End_XY(LCD_XSIZE_TFT-1-i,LCD_YSIZE_TFT-1-i);
  ER_TFT.Circle_Square_Radius_RxRy(10,10);
  ER_TFT.Start_Circle_Square();
  delay(10);
  }

  for(i=0;i<=LCD_XSIZE_TFT/2-10;i+=8)
  {ER_TFT.Foreground_color_65k(Black);
  ER_TFT.Line_Start_XY(0+i,0+i);
  ER_TFT.Line_End_XY(LCD_XSIZE_TFT-1-i,LCD_YSIZE_TFT-1-i);
  ER_TFT.Circle_Square_Radius_RxRy(10,10);
  ER_TFT.Start_Circle_Square();
  delay(10);
  }
  delay(100);

///////////////////////////Circle
  ER_TFT.Foreground_color_65k(Black);
  ER_TFT.Line_Start_XY(0,0);
  ER_TFT.Line_End_XY(LCD_XSIZE_TFT-1,LCD_YSIZE_TFT-1);
  ER_TFT.Start_Square_Fill();

  for(i=0;i<=LCD_XSIZE_TFT/2-10;i+=8)
  {ER_TFT.Foreground_color_65k(Blue);
  ER_TFT.Circle_Center_XY(LCD_XSIZE_TFT/2,LCD_YSIZE_TFT/2);
  ER_TFT.Circle_Radius_R(i);
  ER_TFT.Start_Circle_or_Ellipse();
  delay(10);
  }

  for(i=0;i<=LCD_XSIZE_TFT/2-10;i+=8)
  {ER_TFT.Foreground_color_65k(Black);
  ER_TFT.Circle_Center_XY(LCD_XSIZE_TFT/2,LCD_YSIZE_TFT/2);
  ER_TFT.Circle_Radius_R(i);
  ER_TFT.Start_Circle_or_Ellipse();
  delay(10);
  }
  delay(100);

///////////////////////////Ellipse
  ER_TFT.Foreground_color_65k(Black);
  ER_TFT.Line_Start_XY(0,0);
  ER_TFT.Line_End_XY(LCD_XSIZE_TFT-1,LCD_YSIZE_TFT-1);
  ER_TFT.Start_Square_Fill();

  for(i=0;i<=LCD_XSIZE_TFT/2-100;i+=8)
  {ER_TFT.Foreground_color_65k(White);
  ER_TFT.Circle_Center_XY(LCD_XSIZE_TFT/2,LCD_YSIZE_TFT/2);
  ER_TFT.Ellipse_Radius_RxRy(i+100,i);
  ER_TFT.Start_Circle_or_Ellipse();
  delay(10);
  }

  for(i=0;i<=LCD_XSIZE_TFT/2-100;i+=8)
  {ER_TFT.Foreground_color_65k(Black);
  ER_TFT.Circle_Center_XY(LCD_XSIZE_TFT/2,LCD_YSIZE_TFT/2);
  ER_TFT.Ellipse_Radius_RxRy(i+100,i);
  ER_TFT.Start_Circle_or_Ellipse();
  delay(10);
  }
  delay(100);

 ////////////////////////////Triangle
  ER_TFT.Foreground_color_65k(Black);
  ER_TFT.Line_Start_XY(0,0);
  ER_TFT.Line_End_XY(LCD_XSIZE_TFT-1,LCD_YSIZE_TFT-1);
  ER_TFT.Start_Square_Fill();

  for(i=0;i<=LCD_XSIZE_TFT/2-10;i+=8)
  {ER_TFT.Foreground_color_65k(Yellow);
  ER_TFT.Triangle_Point1_XY(LCD_XSIZE_TFT/2,i);
  ER_TFT.Triangle_Point2_XY(i,LCD_YSIZE_TFT-1-i);
  ER_TFT.Triangle_Point3_XY(LCD_XSIZE_TFT-1-i,LCD_YSIZE_TFT-1-i);
  ER_TFT.Start_Triangle();
  delay(10);
  }

  for(i=0;i<=LCD_XSIZE_TFT/2-10;i+=8)
  {ER_TFT.Foreground_color_65k(Black);
  ER_TFT.Triangle_Point1_XY(LCD_XSIZE_TFT/2,i);
  ER_TFT.Triangle_Point2_XY(i,LCD_YSIZE_TFT-1-i);
  ER_TFT.Triangle_Point3_XY(LCD_XSIZE_TFT-1-i,LCD_YSIZE_TFT-1-i);
  ER_TFT.Start_Triangle();
  delay(10);
  }
  delay(100);


 ////////////////////////////line
  ER_TFT.Foreground_color_65k(Black);
  ER_TFT.Line_Start_XY(0,0);
  ER_TFT.Line_End_XY(LCD_XSIZE_TFT-1,LCD_YSIZE_TFT-1);
  ER_TFT.Start_Square_Fill();

  for(i=0;i<=LCD_XSIZE_TFT;i+=8)
  {ER_TFT.Foreground_color_65k(Red);
  ER_TFT.Line_Start_XY(i,0);
  ER_TFT.Line_End_XY(LCD_XSIZE_TFT-1-i,LCD_YSIZE_TFT-1);
  ER_TFT.Start_Line();
  delay(10);
  }
  for(i=0;i<=LCD_YSIZE_TFT;i+=8)
  {ER_TFT.Foreground_color_65k(Red);
  ER_TFT.Line_Start_XY(0,LCD_YSIZE_TFT-1-i);
  ER_TFT.Line_End_XY(LCD_XSIZE_TFT-1,i);
  ER_TFT.Start_Line();
  delay(10);
  }


  for(i=0;i<=LCD_XSIZE_TFT;i+=8)
  {ER_TFT.Foreground_color_65k(Black);
  ER_TFT.Line_Start_XY(i,0);
  ER_TFT.Line_End_XY(LCD_XSIZE_TFT-1-i,LCD_YSIZE_TFT-1);
  ER_TFT.Start_Line();
  delay(10);
  }
  for(i=0;i<=LCD_YSIZE_TFT;i+=8)
  {ER_TFT.Foreground_color_65k(Black);
  ER_TFT.Line_Start_XY(0,LCD_YSIZE_TFT-1-i);
  ER_TFT.Line_End_XY(LCD_XSIZE_TFT-1,i);
  ER_TFT.Start_Line();
  delay(10);
  }


  delay(100);  

/////////////Internal characters 
  ER_TFT.Select_Main_Window_16bpp();
  ER_TFT.Main_Image_Start_Address(layer1_start_addr);				
  ER_TFT.Main_Image_Width(LCD_XSIZE_TFT);
  ER_TFT.Main_Window_Start_XY(0,0);
  ER_TFT.Canvas_Image_Start_address(layer1_start_addr);
  ER_TFT.Canvas_image_width(LCD_XSIZE_TFT);
  ER_TFT.Active_Window_XY(0,0);
  ER_TFT.Active_Window_WH(LCD_XSIZE_TFT,LCD_YSIZE_TFT);	
 
  
  ER_TFT.Foreground_color_65k(Black);
  ER_TFT.Line_Start_XY(0,0);
  ER_TFT.Line_End_XY(LCD_XSIZE_TFT-1,LCD_YSIZE_TFT-1);
  ER_TFT.Start_Square_Fill();



  ER_TFT.Background_color_65k(Black);
  ER_TFT.Foreground_color_65k(Red);  
  ER_TFT.CGROM_Select_Internal_CGROM();
  ER_TFT.Font_Select_8x16_16x16();
    ER_TFT.Font_Width_X1(); 
  ER_TFT.Font_Height_X1();
  ER_TFT.Goto_Text_XY(0,10);
  ER_TFT.Show_String("buydisplay.com");
  
  ER_TFT.Foreground_color_65k(Green);
  ER_TFT.Font_Select_12x24_24x24();
  ER_TFT.Goto_Text_XY(0,26);
  ER_TFT.Show_String("buydisplay.com");

  ER_TFT.Foreground_color_65k(Blue);
  ER_TFT.Font_Select_16x32_32x32();
  ER_TFT.Goto_Text_XY(0,50);
  ER_TFT.Show_String("buydisplay.com");  
  
  ER_TFT.Foreground_color_65k(Yellow);
  ER_TFT.Font_Width_X4(); 
  ER_TFT.Font_Height_X4();
  ER_TFT.Goto_Text_XY(0,90);  
  ER_TFT.Show_String("buydisplay.com");    
  delay(2000); 
  
       unsigned long im;  
	 for(im=0;im<4;im++)
	 {
        ER_TFT.DMA_24bit_Block(1,0,0,0,LCD_XSIZE_TFT,LCD_YSIZE_TFT,LCD_XSIZE_TFT,im*LCD_XSIZE_TFT*LCD_YSIZE_TFT*2);
        // Select SPI : SCS��0       SCS��1
        // SPI Clock = System Clock /{(Clk+1)*2}
        // Transfer to SDRAM address:X1
         // Transfer to SDRAM address:Y1
        // DMA data width
        // DMA data height
        // Picture's width
        // Flash address	
	  delay(2000);
 	
	 }       
}



